#include "stdafx.h"
#include "OpenGLControl.h"
#include ".\openglcontrol.h"

const float DEG2RAD = 3.14159/180;

COpenGLControl::COpenGLControl(void)
{
	m_fPosX = 0.0f;		// X position of model in camera view
	m_fPosY = 0.0f;		// Y position of model in camera view
	m_fZoom = 10.0f;	// Zoom on model in camera view
	m_fRotX = 0.0f;		// Rotation on model in camera view
	m_fRotY	= 0.0f;		// Rotation on model in camera view
	m_bIsMaximized = false;
}

COpenGLControl::~COpenGLControl(void)
{
}

BEGIN_MESSAGE_MAP(COpenGLControl, CWnd)
	ON_WM_PAINT()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONUP()
END_MESSAGE_MAP()

void COpenGLControl::OnPaint()
{
	ValidateRect(NULL);
}

void COpenGLControl::OnSize(UINT nType, int cx, int cy)
{
	CWnd::OnSize(nType, cx, cy);

	if (0 >= cx || 0 >= cy || nType == SIZE_MINIMIZED) return;

	glViewport(0, 0, cx, cy);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0f,1,0.0f,1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	switch (nType)
	{
		case SIZE_MAXIMIZED:
		{
			GetWindowRect(m_rect);
			GetWindowRect(m_rect);
			m_oldWindow = m_rect;
			break;
		}

		case SIZE_RESTORED:
		{
			if (m_bIsMaximized)
			{
				GetWindowRect(m_rect);
				MoveWindow(m_oldWindow.left, m_oldWindow.top - 18, m_originalRect.Width() - 4, m_originalRect.Height() - 4);
				GetWindowRect(m_rect);
				m_oldWindow = m_rect;
			}
			break;
		}
	}
}

int COpenGLControl::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWnd::OnCreate(lpCreateStruct) == -1) return -1;
	oglInitialize();
	return 0;
}

void COpenGLControl::OnTimer(UINT nIDEvent)
{
	switch (nIDEvent)
	{
		case 1:
		{
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
			oglDrawScene();
			SwapBuffers(hdc);
			break;
		}
		default:
			break;
	}
	CWnd::OnTimer(nIDEvent);
}

void COpenGLControl::OnMouseMove(UINT nFlags, CPoint point)
{
	m_pCurrentPoint = ClientToOpenGL(point);
	CWnd::OnMouseMove(nFlags, point);
}

void COpenGLControl::oglCreate(CRect rect, CWnd *parent)
{
	CString className = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW | CS_OWNDC, NULL, (HBRUSH)GetStockObject(BLACK_BRUSH), NULL);
	CreateEx(0, className, "OpenGL", WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN, rect, parent, 0);
	m_oldWindow	   = rect;
	m_originalRect = rect;
	hWnd = parent;
	m_iLineCount = 0;
	m_iPoliLineCount = 0;
	m_tmpPointCount = 0;
	m_iRectCount = 0;
	m_iCircleCount = 0;
	m_bInProgress = FALSE;
	m_iDrawingMode = DM_EMPTY;
	m_tmpPoliLinePoints = (COpenGLPoint*) calloc(MAX_POLILINE_LONG, sizeof(COpenGLPoint));
	m_Lines =(COpenGLLine*) calloc(MAX_LINE_COUNT, sizeof(COpenGLLine));
	m_PoliLines = (COpenGLPoliLine*) calloc(MAX_POLILINE_COUNT, sizeof(COpenGLPoliLine));
	m_Rects = (COpenGLRect*) calloc(MAX_RECT_COUNT, sizeof(COpenGLRect));
	m_Circles = (COpenGLCircle*) calloc(MAX_CIRCLE_COUNT, sizeof(COpenGLCircle));

	//m_Circles = (CCircle*) calloc(MAX_CIRCLE_COUNT, sizeof(CCircle));
}

void COpenGLControl::oglInitialize(void)
{
	// Initial Setup:
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		32, // bit depth
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
		16, // z-buffer depth
		0, 0, 0, 0, 0, 0, 0,
	};

	hdc = GetDC()->m_hDC;
	// Pixel format.
	m_nPixelFormat = ChoosePixelFormat(hdc, &pfd);
	SetPixelFormat(hdc, m_nPixelFormat, &pfd);
	// Create the OpenGL Rendering Context.
	hrc = wglCreateContext(hdc);
	wglMakeCurrent(hdc, hrc);
	// Set color to use when clearing the background.
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0f);
	// Turn on backface culling
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);
	// Turn on depth testing
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
}

void COpenGLControl::oglDrawScene(void)
{
	float r;
	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

	//Draw lines
	for(int i = 0; i < m_iLineCount; i++)
	{
		glColor3fv(m_Lines[i].m_Color);
		glBegin(GL_LINES);
			glVertex2f(m_Lines[i].m_pLeft.x, m_Lines[i].m_pLeft.y);
			glVertex2f(m_Lines[i].m_pRight.x, m_Lines[i].m_pRight.y);	
		glEnd();
	}
	//Draw rects
	for(int i = 0; i < m_iRectCount; i++)
	{
		glColor3fv(m_Rects[i].m_Color);
		glBegin(GL_QUADS);
			glVertex2f(m_Rects[i].m_pUpLeft.x, m_Rects[i].m_pUpLeft.y);
			glVertex2f(m_Rects[i].m_pDownRight.x, m_Rects[i].m_pUpLeft.y);	
			glVertex2f(m_Rects[i].m_pDownRight.x, m_Rects[i].m_pDownRight.y);	
			glVertex2f(m_Rects[i].m_pUpLeft.x, m_Rects[i].m_pDownRight.y);	
		glEnd();
	}
	//Draw poligons
	for(int i = 0; i < m_iPoliLineCount; i++)
	{
		glColor3fv(m_PoliLines[i].m_Color);
		if(m_PoliLines[i].m_bClosed)  glBegin(GL_LINE_LOOP);
		if(!m_PoliLines[i].m_bClosed) glBegin(GL_LINE_STRIP);
			for(int j = 0; j < m_PoliLines[i].m_pCount; j++)
			{
				glVertex2f(m_PoliLines[i].m_pArray[j].x,m_PoliLines[i].m_pArray[j].y);
			}
		glEnd();
	}
	for(int i = 0; i < m_iCircleCount; i++)
	{
		glColor3fv(m_Circles[i].m_Color);
		glBegin(GL_LINE_LOOP);
 			for (int j=0; j < 360; j++)
			{
				float degInRad = j*DEG2RAD;
				glVertex2f(cos(degInRad)*m_Circles[i].r+m_Circles[i].m_pCenter.x,
						   sin(degInRad)*m_Circles[i].r+m_Circles[i].m_pCenter.y);
			}
		glEnd();
	}

	if(m_bInProgress)
	{
		glColor3fv(m_CurrentColor);
		switch(m_iDrawingMode)
		{
		case DM_LINE:
			glBegin(GL_LINES);
				glVertex2f(m_pStartPoint.x,m_pStartPoint.y);
				glVertex2f(m_pCurrentPoint.x, m_pCurrentPoint.y);	
			glEnd();
			break;
		case DM_RECT:
			glBegin(GL_QUADS);
				glVertex2f(m_pStartPoint.x,m_pStartPoint.y);
				glVertex2f(m_pCurrentPoint.x,m_pStartPoint.y);
				glVertex2f(m_pCurrentPoint.x, m_pCurrentPoint.y);	
				glVertex2f(m_pStartPoint.x, m_pCurrentPoint.y);	
			glEnd();
			break;
		case DM_CIRCLE:
			r = (float)sqrt((m_pStartPoint.x-m_pCurrentPoint.x)*(m_pStartPoint.x-m_pCurrentPoint.x)
								 +(m_pStartPoint.y-m_pCurrentPoint.y)*(m_pStartPoint.y-m_pCurrentPoint.y));
			glBegin(GL_LINE_LOOP);
 				for (int i=0; i < 360; i++)
				{
					float degInRad = i*DEG2RAD;
					glVertex2f(cos(degInRad)*r+m_pStartPoint.x,
							   sin(degInRad)*r+m_pStartPoint.y);
				}
				glEnd();
			break;
		case DM_LINELOOP:
		case DM_LINESTRIP:
			if(m_iDrawingMode==DM_LINELOOP)  glBegin(GL_LINE_LOOP);
			if(m_iDrawingMode==DM_LINESTRIP) glBegin(GL_LINE_STRIP);
				for(int i = 0; i < m_tmpPointCount; i++)
				{
					glVertex2f(m_tmpPoliLinePoints[i].x, m_tmpPoliLinePoints[i].y);
				}
				glVertex2f(m_pCurrentPoint.x,m_pCurrentPoint.y);
			glEnd();
			break;
		default:
			break;
		}
	}

}

void COpenGLControl::SetDrawingMode(int dMode)
{
	m_iDrawingMode = dMode;
	m_tmpPointCount = 0;
}

void COpenGLControl::OnLButtonUp(UINT nFlags, CPoint point)
{
	m_pStartPoint = ClientToOpenGL(point);
	switch(m_iDrawingMode)
	{
		case DM_LINE:
		case DM_RECT:
		case DM_CIRCLE:
			m_bInProgress = m_bInProgress?FALSE:TRUE;
			break;
		case DM_LINELOOP:
		case DM_LINESTRIP:
			m_bInProgress = TRUE;
			m_tmpPoliLinePoints[m_tmpPointCount] = ClientToOpenGL(point);
			m_tmpPointCount++;
	}
	CWnd::OnLButtonUp(nFlags, point);
}

COpenGLPoint COpenGLControl::ClientToOpenGL(CPoint point)
{
	COpenGLPoint glPoint;
	glLoadIdentity();
	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	GLdouble posX, posY, posZ;

	glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
	glGetDoublev( GL_PROJECTION_MATRIX, projection );
	glGetIntegerv( GL_VIEWPORT, viewport );

	gluUnProject((GLdouble)point.x, (GLdouble)(viewport[3]-point.y), (GLdouble)0, modelview, projection, viewport, &posX, &posY, &posZ);

	glPoint.x = posX;
	glPoint.y = posY;
	glPoint.z = posZ;

	return glPoint;
}

void COpenGLControl::OnRButtonUp(UINT nFlags, CPoint point)
{
	COpenGLPoint p;
	if(m_bInProgress)
	{
		m_bInProgress = FALSE;
		switch(m_iDrawingMode)
		{
		case DM_LINE:
			m_Lines[m_iLineCount].m_pLeft = m_pStartPoint;
			m_Lines[m_iLineCount].m_pRight = ClientToOpenGL(point);
			m_Lines[m_iLineCount].m_Color = m_CurrentColor;
			m_iLineCount++;
			break;
		case DM_RECT:
			m_Rects[m_iRectCount].m_pUpLeft = m_pStartPoint;
			m_Rects[m_iRectCount].m_pDownRight = ClientToOpenGL(point);
			m_Rects[m_iRectCount].m_Color = m_CurrentColor;
			m_iRectCount++;
			break;
		case DM_CIRCLE:
			m_Circles[m_iCircleCount].m_pCenter = m_pStartPoint;
			p = ClientToOpenGL(point);
			m_Circles[m_iCircleCount].r = sqrt((m_pStartPoint.x-p.x)*(m_pStartPoint.x-p.x)
											  +(m_pStartPoint.y-p.y)*(m_pStartPoint.y-p.y));
			m_Circles[m_iCircleCount].m_Color = m_CurrentColor;
			m_iCircleCount++;
			break;
		case DM_LINELOOP:
		case DM_LINESTRIP:
			m_tmpPoliLinePoints[m_tmpPointCount] = ClientToOpenGL(point);
			m_tmpPointCount++;
			m_PoliLines[m_iPoliLineCount].m_pCount = m_tmpPointCount;
			m_PoliLines[m_iPoliLineCount].m_bClosed = (m_iDrawingMode == DM_LINELOOP);
			m_PoliLines[m_iPoliLineCount].m_pArray =  
				(COpenGLPoint*) calloc(m_tmpPointCount, sizeof(COpenGLPoint));
			for(int i = 0; i < m_tmpPointCount; i++)
			{
				m_PoliLines[m_iPoliLineCount].m_pArray[i] = m_tmpPoliLinePoints[i];
			}
			m_PoliLines[m_iPoliLineCount].m_Color = m_CurrentColor;
			m_tmpPointCount = 0;
			m_iPoliLineCount++;
			break;
		default:
			break;
		}
	}
	m_tmpPointCount = 0;
	CWnd::OnRButtonUp(nFlags, point);
}

void COpenGLControl::SetCurrentColor(float * color)
{
	m_CurrentColor = color;
}